<?php

/**
 * @file
 * Contains \Drupal\taxonomy_container\Plugin\Field\widget\SelectWithGroupsWidget.
 */

namespace Drupal\taxonomy_container\Plugin\Field\FieldWidget;

use Drupal\Core\Field\Plugin\Field\FieldWidget\OptionsSelectWidget;
use Drupal\Core\Entity\FieldableEntityInterface;
use Drupal\taxonomy\Plugin\Field\FieldType;

/**
 * Plugin implementation of the 'select_with_groups' widget.
 *
 * @FieldWidget(
 *   id = "select_with_groups",
 *   module = "taxonomy_container",
 *   label = @Translation("Select list with groups"),
 *   field_types = {
 *     "taxonomy_term_reference",
 *   },
 *   multiple_values = TRUE
 * )
 */
class SelectWithGroupsWidget extends OptionsSelectWidget {

  /**
   * Returns the array of options for the widget.
   *
   * @param \Drupal\Core\Entity\FieldableEntityInterface $entity
   *   The entity for which to return options.
   *
   * @return array
   *   The array of options for the widget.
   */
  protected function getOptions(FieldableEntityInterface $entity) {
    if (!isset($this->options)) {

      $options_provider = $this->fieldDefinition
        ->getFieldStorageDefinition()
        ->getOptionsProvider($this->column, $entity);

      // See https://www.drupal.org/node/2351551.
      $allowed_values = $options_provider->getFieldDefinition()->getSetting('allowed_values');

      $options = array();
      foreach ($allowed_values as $tree) {
        if ($vocabulary = entity_load('taxonomy_vocabulary', $tree['vocabulary'])) {
          $terms = \Drupal::entityManager()
            ->getStorage('taxonomy_term')->loadTree($vocabulary->id(), $tree['parent'], NULL, TRUE);
          if ($terms) {
            foreach ($terms as $term) {
              if ($term->parents[0] == 0) {
                $parent_name = $term->label();
              }
              else {
                $options[$parent_name][$term->id()] = str_repeat('-', $term->depth) . $term->label();
              }
            }
          }
        }
      }

      // Add an empty option if the widget needs one.
      if ($empty_option = $this->getEmptyOption()) {
        $label = ($empty_option == static::OPTIONS_EMPTY_NONE ? t('- None -') : t('- Select a value -'));
        $options = array('_none' => $label) + $options;
      }

      $module_handler = \Drupal::moduleHandler();
      $context = array(
        'fieldDefinition' => $this->fieldDefinition,
        'entity' => $entity,
      );
      $module_handler->alter('options_list', $options, $context);

      array_walk_recursive($options, array($this, 'sanitizeLabel'));

      $this->options = $options;
    }

    return $this->options;
  }

}
